#include "VertexArray.h"
#include "VertexBuffer.h"
#include "Renderer.h"
#include "VertexBufferLayout.h"

VertexArray::VertexArray()
{
  GLCall(glGenVertexArrays(1, &m_RendererID));
  if (!m_RendererID) {
    std::cerr << "Failed to generate VAO" << std::endl;
  }


}
VertexArray::~VertexArray()
{
  GLCall(glDeleteVertexArrays(1, &m_RendererID));
}

void VertexArray::AddBuffer(const VertexBuffer& vb, const VertexBufferLayout& layout)
{
  Bind();
  vb.Bind();
  const auto& elements = layout.GetElements();
  GLsizei offset = 0;
  for (unsigned int i = 0; i < elements.size(); i++) {
    const auto& element = elements[i];
    GLCall(glEnableVertexAttribArray(i));
    GLCall(glVertexAttribPointer(i, element.count, element.type, element.normalized, layout.GetStride(), (const void*)offset));
    offset += element.count * VertexBufferElement::GetSizeOfType(element.type);
    }
}

void VertexArray::Bind() const {
  GLCall(glBindVertexArray(m_RendererID));
}
void VertexArray::Unbind() const {
  GLCall(glBindVertexArray(0));
}
